// Copyright (c) 2005  Hitoshi Guutara Maruyama.
// This is free software;  for terms and warranty disclaimer see ./COPYING.

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.ejb.EJB;

import jp.sourceforge.gnp.dsearch.ejb.interfaces.GetDataEjb;

/**
 * class <code>GetDataClientEjb</code>
 *
 * @author <a href="mailto:maruyama@sh.rim.or.jp">Hitoshi Guutara Maruyama</a>
 * @version 1.0
 */
class GetDataClientEjb {

  static final String	DEFAULT_DATE = "20060401";

  static String	getDate = DEFAULT_DATE;

  /**
   * variable <code>getData</code>	GetDataEjb EJB3 object
   *
   */
  @EJB
    GetDataEjb	getData = null;

  GetDataClientEjb() throws Exception {
    super();
    Context context = null;
    try {
      context = new InitialContext();
    } catch (NamingException e) {
      e.printStackTrace();
      throw e;
    }
    Object ref = null;
    try {
      ref = context.lookup(GetDataEjb.JNDI_NAME);
    } catch (NamingException e) {
      e.printStackTrace();
      throw e;
    }
    getData = (GetDataEjb)ref;
  }
  
  public static void	main(String [] args) {
    if (args.length < 2) {
      System.err.println("usage : java GetDataClientEjb command(factor|city|airway|classes) arg [date]");
      return;
    }
    String	command = args[0];
    String	arg = args[1];
    GetDataClientEjb	client = null;
    try {
      client = new GetDataClientEjb();
    }
    catch (Exception e) {
      System.err.println("error in creating GetDataClientEjb() : "
			 + e.getMessage());
    }
    if (client == null) {
      return;
    }

    GetDataEjb	getData = null;
    try {
      getData = client.getData;
    }
    catch (Exception e) {
      System.err.println("error in get getData from GetDataClientEjb instance : "
			 + e.getMessage());
      return;
    }
    
    if (command.equals("factor")) {
      if (args.length < 3) {
	System.err.println("usage : java GetDataClientEjb factor from to [date]");
	return;
      }
      String	from = args[1];
      String	to = args[2];
      if (args.length > 3) {
	getDate = args[3];
      }
      double retval = (double)0;
      try {
	retval = getData.getProrateFactor(from, to, getDate);
      }
      catch (Exception e) {
	System.err.println("error in getProrateFactor for "
			   + from + "-" + to + " : " + e.getMessage());
	return;
      }
      System.out.println("getProrateFactor for "
			 + from + "-" + to + " : " + retval);
    }
    else if (command.equals("city")) {
      if (args.length < 2) {
	System.err.println("usage : java GetDataClientEjb city arg");
	return;
      }
      String retval = null;
      try {
	retval = getData.getCityName(arg);
      }
      catch (Exception e) {
	System.err.println("error in getCityName for "
			   + arg + " : " + e.getMessage());
	return;
      }
      System.out.println("getCityName for " + arg + " : " + retval);
    }
    else if (command.equals("airway")) {
      if (args.length < 2) {
	System.err.println("usage : java GetDataClientEjb airway arg [date]");
	return;
      }
      if (args.length > 2) {
	getDate = args[2];
      }
      String retval = null;
      try {
	retval = getData.getAirwayId(arg, getDate);
      }
      catch (Exception e) {
	System.err.println("error in getAirwayId for "
			   + arg + " : " + e.getMessage());
	return;
      }
      System.out.println("getAirwayId for " + arg + " : " + retval);
    }
    else if (command.equals("classes")) {
      if (args.length < 2) {
	System.err.println("usage : java GetDataClientEjb classes arg");
	return;
      }
      String retval = null;
      try {
	retval = getData.getClassesForMainClass(arg);
      }
      catch (Exception e) {
	System.err.println("error in getClassesForMainClass for "
			   + arg + " : " + e.getMessage());
	return;
      }
      System.out.println("getClassesForMainClass for " + arg + " : " + retval);
    }
  }

}
