// Copyright (c) 2002-2009  Hitoshi Guutara Maruyama.
// This is free software;  for terms and warranty disclaimer see ./COPYING.

package jp.sourceforge.gnp.prorate.struts.service;


import jp.sourceforge.gnp.prorate.export.ProrateAudit;
import jp.sourceforge.gnp.prorate.jdl.ProrateClient;
import jp.sourceforge.gnp.prorate.struts.view.ProrateView;

/**
 * Describe class <code>ProrateServiceJdl</code> here.
 *	struts proration service using remote prorate client
 *
 * @author <a href="mailto:gnp@sourceforge.jp">Hitoshi Guutara Maruyama</a>
 * @version 1.0
 */
public class ProrateServiceJdl extends ProrateService {

  // Constructor
  /**
   * Creates a new <code>ProrateServiceJdl</code> instance.
   *
   * @exception Exception if an error occurs
   */
  public ProrateServiceJdl() throws Exception {
    super();
  }

  // Operations
  /**
   * <code>prorate</code> method
   *	creates prorate rmi client object,
   *	requests a prorate calculation to the rmi proration server.
   *	creates prorate view, sets data from the proration result,
   *	and returns it.
   *
   * @param audit a <code>ProrateAudit</code> value
   * @return a <code>ProrateView</code> value
   */
  public ProrateView prorate(ProrateAudit audit) {
    ProrateAudit	auditRet = null;
    ProrateClient	prorate = null;
    try {
      prorate = createProrate();
    }
    catch (Exception e) {
      e.printStackTrace();
      ProrateView	errorView = createErrorView(audit);
      errorView.setFatalError(true);
      errorView.setErrorString(e.getMessage());
      if (errorView.getErrorString() == null) {
	errorView.setErrorString(e.toString());
      }
      return errorView;
    }
    try {
      auditRet = prorate.prorate(audit);
    }
    catch (Exception e) {
      e.printStackTrace();
      ProrateView	errorView = createErrorView(audit);
      errorView.setFatalError(true);
      errorView.setErrorString(e.getMessage());
      if (errorView.getErrorString() == null) {
	errorView.setErrorString(e.toString());
      }
      return errorView;
    }
    ProrateView	prorateView = createView(auditRet);
    return prorateView;
  }

  /**
   * <code>createProrate</code> method
   *	creates prorate rmi client object and returns it.
   *
   * @return a <code>ProrateClient</code> value
   * @exception Exception if an error occurs
   */
  ProrateClient	createProrate() throws Exception {
    ProrateClient	prorate = new ProrateClient(System.getProperties());
    return prorate;
  }
}
