// Copyright (c) 2002  Hitoshi Guutara Maruyama.
// This is free software;  for terms and warranty disclaimer see ./COPYING.

package jp.sourceforge.gnp.prubae;


import java.util.List;

import jp.sourceforge.gnp.rulebase.ProrateRulebaseWriter;

/**
 * class <code>PrubaeModelJudge</code>
 * is the common super class of prubae judge statements rule element models,
 * such as and/or-statement and other judge statements such as = Another() etc.
 * @author <a href="mailto:gnp@sourceforge.jp">Hitoshi Guutara Maruyama</a>
 * @version 1.0
 */
public class PrubaeModelJudge extends PrubaeModel {

  // Constructor  
  /**
   * Creates a new <code>PrubaeModelJudge</code> instance.
   * this creates all GUI elements on part panel of the judge statement.
   */
  public PrubaeModelJudge() {  
    super();  

    setView(new PrubaeViewJudge());
    getView().setModel(this);
    setController(new PrubaeControllerJudge());
    getController().setModel(this);
    getView().setController(getController());
    getController().setView(getView());
  }  

  // Operations  
  /**
   * <code>swap</code> method
   * swaps this model and the old model as the if-statement test condition.
   * @param old a <code>PrubaeModel</code> value
   */
  public void swap(PrubaeModel old) {
    List	vec = old.getParentList();
    if (vec == null) {
      initialize(old.getEditor(), old.getParent(), vec);
      PrubaeModelIf	oldIf = (PrubaeModelIf)old.getParent();
      oldIf.setCondition(this);
    }
    else {
      super.swap(old);
    }
  }  

  /**
   * <code>toString</code> method
   * returns comment string if exists, or returns "new judgement" otherwise.
   * @return a <code>String</code> value
   */
  public String toString() {  
    if (getComment() != null && !getComment().equals("")) {
      return getComment();
    }
    return "new judgement";
  }  

  /**
   * <code>writeModel</code> method writes this rule element model.
   * this actually does nothing.
   */
  public void writeModel(ProrateRulebaseWriter rulebase) {
    rulebase.writeJudge(this);
  }

  /**
   * <code>getSize</code> method
   * counts the size of this rule and returns the size which is always 0.
   * @param rulebase a <code>ProrateRulebaseWriter</code> value
   * @return an <code>int</code> value
   */
  public int getSize(ProrateRulebaseWriter rulebase) {
    return 0;
  }

  /* ;;; 2006.09.12 */
  /**
   * <code>check</code> method	check validity of the prubae model.
   *
   * @param errors a <code>List</code> value
   * @return a <code>boolean</code> value
   */
  public boolean check(List errors) {
    return true;
  }

} /* end class PrubaeModelJudge */
